'use strict'

const _ = require('lodash')
const request = require('request')
// require('request-debug')(request)

const config = require('./rc')().get()
const auth = config.medx.auth
const c3 = config.medx.c3

const no = ['no', 'none', 'nope', 'nein']
const abortSearch = (term) => _.includes(no, term.trim().toLowerCase())

const convertToDate = (date) => {
  const from = date.trim().split('.')
  return new Date(from[2], from[1] -1, from[0]).toISOString()
}

module.exports = (username, password) => {
  let api = null

  const loginFormData = {
    username: username,
    password: password
  }

  const initialFormData = JSON.stringify({
    meta: { state: 'DASHBOARD' }
  })

  const getResult = (props, cb) => {
    api.get({url: `result/${props.selfAssessmentKey}`}, (err, res, body) => {
      if (err) cb(err)
      cb(null, JSON.parse(body))
    })
  }

  const normalizeResponse = (response) => {
    const firstType = _.get(response, 'answerOptions[0].type')
    const secondType = _.get(response, 'answerOptions[1].type')

    response.answerType = firstType === 'SELECT' ? 'SELECT' : 'TEXT'
    response.search = firstType === 'SEARCH'

    if (firstType === 'SEARCH' && secondType) {
      response.question += ' Type "no" or enter a symptom.'
    }
    if (firstType === 'DATE') {
      response.question += ' e.g.: 19.07.1994'
    }
    return response
  }

  const symptomSearch = (answer, previousMessage, cb) => {
    api.post({
      url: `/dialog/search?limit=5&offset=0&query=${encodeURIComponent(answer)}`,
      body: JSON.stringify({meta: previousMessage.meta})},
      (err, res, body) => {
        const results = JSON.parse(body).results
        let newMessage = {
          question: 'Which descibes your pain best?',
          answerType: 'SELECT',
          meta: previousMessage.meta,
          answerOptions: results
        }
        if (!results.length) console.log('No results were found.')
        cb(err, !err && results.length ? newMessage : previousMessage)
      })
  }

  return {
    login: (cb) => {
      request.post({url: auth, formData: loginFormData}, (err, res) => {
        if (err) cb(err)
        const cookie = res.headers['set-cookie'][0].match(/([^=]+=[^;]+)/)[1]

        api = request.defaults({
          baseUrl: c3,
          headers: {
            cookie,
            'Content-Type': 'application/json;charset=UTF-8'
          }
        })

        api.post({url: '/dialog', body: initialFormData}, (err, res, body) => {
          cb(err, !err && normalizeResponse(JSON.parse(body)))
        })
      })
    },

    isLoggedIn: () => !!api,

    ask: (answer, previousMessage, cb) => {
      if (!api) cb(new Error('wrong function'))

      if (previousMessage.search && !abortSearch(answer)) {
        return symptomSearch(answer, previousMessage, cb)
      }

      let dialogFormData = {
        meta: previousMessage.meta,
        answer: answer
      }

      if (previousMessage.answerType === 'TEXT') {
        const type = previousMessage.answerOptions[0].type

        answer = answer.trim()
        dialogFormData.answer = abortSearch(answer) ? {key: 'NO'}
        : {key: answer, text: type === 'DATE' ? convertToDate(answer) : answer}
      }

      if (answer.score) dialogFormData.answerType = 'SEARCH'

      dialogFormData = JSON.stringify(dialogFormData)

      api.post({url: '/dialog', body: dialogFormData}, (err, res, body) => {
        body = JSON.parse(body)
        if (body.answerOptions[0].type === 'NAVIGATE') {
          return getResult(body.answerOptions[0].props, cb)
        }
        cb(err, !err && normalizeResponse(body))
      })
    }
  }
}
